% function SolveSubproblem includes several parts: calculating costs-to-go
% for terminal states; calculating costs for all routes and derive the
% guidance; computing the group's route choice behaviors: all routes
% considered and associated probabilities; calculating expected flow rates
% and system dynamics; updating lambda, c, alpha, Beta, s  
%
function [x,qd_sum,Stepsize_tt,gnorm_tt,lambda_tt,alpha_tt,ck1,q_tt,AllRoute,routeV]=SolveSubproblem(v_I,C,BGobj,groupIndex,ke,routeV,RouteInMind,To,SrouteV,EdgeV, StayC,Blc,alpha,betaF,betaG, betaH,betaV, betaB, betaGV,betaHV,betaHS,risk,lambdaV_t,lambda_t,GroupO,CapaN, CapaR,r1, Beta, M, Stepsize_t, gnorm_t,alpha_t, ck, q_t, iter_num,AllRoute,x,Visibility,Zs,Num_oV,qd_sum)
% Old "function:"
% function [routeV, pathV, CostR, CostB]=SolveSubproblem(v_I,groupN, groupIndex,Num_oV, C, BGobj, ke,routeV, 
% pathV, RouteInMind, To, SrouteV,EdgeV, StayC,betaG, betaH, betaF, beta 
%
% Inputs of the function: v_I (the starting position of the group),
% C, BGobj (network structure), groupIndex (the index of the group)
% [removed] Num_oV (the number of vertices in the [original] network), groupN (the group size), 
% [removed] pathV,  
% [removed] p_pass (probabilities of passing edges), Block_f (blocked vertices because of fire), 
% ke (the number of exits), routeV (Vectors of the suggested routes), 
% RouteInMind (the routes in occupants' minds [familiar or visible]), 
% To (the lookahead steps), SrouteV (shortest routes to different exits),
% EdgeV (shortest routes to exits), StayC (the risks of staying in rooms),
% (psychological features): betaF,betaG, betaH,betaV, betaB, betaV,betaGV,betaHV,betaHS,
% risk (the risk matrix: fire and smoke),
% (Lagrangian multipliers): lambdaV_t,lambda_t, 
% GroupO (the initial states of all groups)
% CapaN: passage capacities ([previously] include familiarity),
% [added] CapaR: room capacities, 
% [added] (SAVLR related) r1, Beta, M, Stepsize_t, gnorm_t,
% [added] alpha_t, ck, q_t, iter_num (the number of iteration: k)
% [added] AllRoute (the set of routes used by all groups and corresponding probabilities)
% [added] Visibility (the visibilty matrix at each time step)
% [added] routeV (guidance for all groups)
%
% Outputs of the function: routeV ([updated] Vectors of the suggested routes)
% x([updated] the system dynamic in previous iteration), ([updated] Lagrangian multipliers): lambdaV_t,lambda_t,
% [added] (SAVLR related) Stepsize_tt, gnorm_tt, lambda_tt, alpha_tt, ck1,q_tt 
% [added] AllRoute (the probability of using routes by different occupants)
% [added] qd_sum (the expected desired flow rates: calculating the duality gap)
%
% AllRoute(the index of a group, the index of a possible route, T + probability)
% AllRoute=zeros(Group_num,(Num_V+Group_num),(T+1));
%
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%  Terminal Costs (heuristic route)   %%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Input: C, routeV, v_I, To,SrouteV,EdgeV,lambdaV_t,lambda_t,ke,risk,StayC,q_t,ck1
% Function used: FcandiV.m, heuriA.m, 
% outputs: PCostB, PCostR, HrouteV 
% 
[Num_V Num_E]=size(C);
% Cost about blockages at t= To+1(d+1)
PCostB=zeros(Num_V,1); 
% Cost about risks at t= To+1(d+1)
PCostR=zeros(Num_V,1); 
% PCost=zeros(Num_V,1); 
%
% (Heuristic) route: (written as) vertices 
[xx T]=size(routeV);
HrouteV=zeros(Num_V,T); 
VerS=FcandiV(v_I,To,C);
for i=1:Num_V
    if VerS(To+1,i)==0
        break;
    else
        v=VerS(To+1,i);
        %[c OrouteV]=heuriA(VerS(d+1,i),Num_V,SrouteV,EdgeV,lambda,ke,r);
        % "OrouteV": Original route extended from a Vertex
        % the stay strategy has been considered in HeuriA.m 
        % add q_t and ck: the absolute penalty in SAVLR 
        [cB cR OrouteV]=heuriA(v,To,Num_V,SrouteV,EdgeV,lambdaV_t,lambda_t,ke,risk,StayC,q_t,ck);
        PCostB(v)=cB;
        PCostR(v)=cR;
       % PCost(v,To+1)=cB+cR;   % Cost at t = To+1
        HrouteV(v,(To+1):T)=OrouteV(1:(T-To));  % assuming T<Num_V
    end
end
% Is it necessary to compute PCost for all vertices? Yes, because all these
% vertices are possibly suggested in the guidance 
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%   Costs for all routes    %%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Inputs: v_I, To, C, BGobj, ke, 
% Function used: FcandiV.m,  
% outputs: UtilityV, PathV 
%
%% Given v_I, derive VerS
% "t" referred to the time point of encountering blockage (the time point of reselect a route)
% "d" and "To" are equal in the new version; thus, "t" = 0 
t=0;
d= To-t;
% find out all possible state at t = d 
VerS=FcandiV(v_I,d,C);
% The origin of the group 
Origin=VerS(1,1);
% The vector that stores utilities of different routes 
UtilityV=[];
% Storing possible paths: a set of vectors 
PathV=[]; % different from pathV ? (pathV is a vector that temporarily store edges on a route)
%
%% calculating the cost for each possible route
for i=1:Num_V
     %  if VerS(d+1,:) reaches a zero element
    if VerS(d+1,i)==0
        break;
    else
        %  if VerS(d+1,i) is a non-zero element 
        Destination=VerS(d+1,i);
        %% Generate the route
        % if the destination at t=d is out of an exit 
        % recognize the exact index of the exit 
        % find out the corresponding exit (nearest exit)
        if Destination==Num_V
            Dist=zeros(1,ke);
            for i2=1:ke
                Destination=Num_V+i2-1;
                [dist,path1,pred] = shortestpath(BGobj,Origin,Destination,'directed',false,'method','BFS');
                Dist(i2)=dist;
            end
            % find out the nearest exit 
            [c1 e1]=min(Dist);
            Destination=Num_V+e1-1;
            [dist,path1,pred] = shortestpath(BGobj,Origin,Destination,'directed',false,'method','BFS');
            path1(dist+1)=Num_V;
            Destination=Num_V;
            %
        else
            % if the destination at t=d is within the building
            [dist,path1,pred] = shortestpath(BGobj,Origin,Destination,'directed',false,'method','BFS');
            % !!! revised on 10/04/2018
            % if dist < To, additional states should be added
            %
            if dist<To
                path1=[path1 Destination*ones(1,(To-dist))];
            end % all path1 will have the length (1+To)
        end
        % Output: dist is the distance of the route; path1 is the set of vertices
        %
        %% Calculate the utility of the route; store the route 
        %
        Zb=PCostB(Destination);
        Zr=PCostR(Destination);
        if dist==0
            Zr=Zr+ risk(path1(dist+1),(dist+1));
        else
            for i1=1:dist
                % including the risk at t = To (because PCostR has not included that vertex)
                Zr=Zr+ risk(path1(i1+1),i1+1);
                [EdgeN flag]=EdgeF(path1(i1),path1(i1+1),C);
                % the cost of blockage
                if EdgeN~=0
                    Zb = Zb+lambdaV_t(path1(i1+1),i1+1)+lambda_t(EdgeN,i1)+q_t(EdgeN,i1)*ck/2;
                else
                    Zb = Zb+lambdaV_t(path1(i1+1),i1+1);
                end
            end
%             % if dist< To, Zr is underestimated
%             Zr= Zr+risk(path1(i1+1),i1+1)*(To-dist);
        end
        Utility=Zb+Zr;
        UtilityV=[UtilityV;Utility];
        % Store the route
        % unify the length of the route:  Num_V
        % 
        % PathP: path1 with zero-padding
        if length(path1)<Num_V
            Num0=Num_V-length(path1);
            % the dimension of path1: 1*length(path1) 
            PathP=[path1 zeros(1,Num0)];
        end 
        %        
        PathV=[PathV;PathP];
    end
end
%
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%   Selecting the best route (to suggest) %%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Inputs: UtilityV, PathV 
% outputs: Path_guide, Eng_guide
%
%% Selecting the best route (to suggest)
%
[cx xx]=size(GroupO);
% cx is the number of groups; xx has no use;
[Cost k1]=min(UtilityV);
% the suggested route (the partial route in the planning horizon)
% the effective part in Path_guide is 1:(d+1)
Path_guide=PathV(k1,1:(1+To));
%
% The destination of Path_guide
flag=0;
for i=1:(1+To)
    if Path_guide(i)==0
        flag=1;
        break;
    end
end
if flag~=1
    % if Path_guide does not terminate too early
    End_guide=Path_guide(To+1);
else
    End_guide=Path_guide(i-1);
end
%
%% Is the newly-derived guidance identical to the previous guidance? 
% the new guidance: Path_guide; the old guidance: routeV(groupIndex,:)
if isequal(routeV(groupIndex,1:(1+To)),Path_guide)==0
    % is this guidance (Path_guide) different from previous guidance?
    % if no (there is no change on expected dynamics) and check surrogate
    % optimality condition directly; if yes, update the system dynamics and
    % check surrogate optimality condition
    %
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % %%%%%%  Possible selected routes and probabilities  %%%%%%%%%%
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %
    % Inputs: Path_guide, RouteInMind, To (d), C, routeV, groupIndex, 
    % Visibility (Visible_t),
    % Function used: EdgeF.m,
    % outputs: RouteSet (including the route and probability)
    %
    % RouteSet: the set of routes that are considered to select by the group
    % RouteSet includes familiar routes and routes to visible exits (in RouteInMind)
    % RouteSet includes suggested route (guidance)
    RouteSet=[];
    %
    % RouteInMind=zeros((Num_oV),ke,(To+7));
    %
    RouteInMindt=reshape(RouteInMind(v_I,:,:),[ke,(To+1+6)]);
    RouteInMindi=[RouteInMindt(:,1:(d+1)) RouteInMindt(:,(To+2):(To+7))];
    %
    % RouteInMindi(To+2):Zf
    % RouteInMindi(To+3):Zg
    % RouteInMindi(To+4):Zh
    % RouteInMindi(To+5):Zv
    % RouteInMindi(To+6): Destination of the (partial) route
    % RouteInMindi(To+7): Probability of selecting this route (initial value: 1)
    % Jt_r and Jt_b are functions of the vertex at RouteInMindi(To+6)
    % Zs is not included in this vector; defined at IncidenceM
    %
    % RouteInMindi: dimension: ke*(d+1): for a single group
    % remove zero vectors in RouteInMindi
    for i=1:ke
        if RouteInMindi(i,1)~=0
            RouteSet=[RouteSet;RouteInMindi(i,:)];
        else
            continue;
        end
    end
    %
    % is the suggested route in "RouteSet?"
    % RouteSet: include familiar routes, visible routes, suggested routes
    [flag row_n]= ismember(Path_guide,RouteSet(:,1:(d+1)),'rows');
    if flag==1
        % the property for guidance on the suggested route: z_g = 1
        RouteSet(row_n,(To+3))=1;
    else
        % the route is not in RouteSet
        %
        % the guidance end at Vertex End_guide
        newroute=[Path_guide,0,1,0,0,End_guide,1];
        RouteSet=[RouteSet;newroute];
    end
    %
    %% add possible routes affected by herding
    %
    % For each route (already) in RouteSet, at each time step, consider the
    % possible herding routes
    % find the number of routes in RouteSet; "xx" is useless
    [route_num xx]=size(RouteSet);
    %
    % clear the guidance for the current group (not overlap in herding)
    routeV(groupIndex,:)=zeros(1,T);
    %
    for i=1:d
        % NodeSet: the set of nodes at t = i-1
        % NodeSet is introduced to avoid repeat calculation at the same node
        NodeSet=[];
        % the dimension of Visible_t: (Num_V-1)*(Num_V-1)
        Visible_t=reshape(Visibility(i,:,:),[],Num_oV-1);
        % at t=i-1, derive all possible vertices
        %
        for i1=1:route_num
            % if the vertex (node) at t = i-1 has been considered, skip to the
            % next vertex; otherwise, calculate the probabilities of route
            % choices
            if ismember(RouteSet(i1,i),NodeSet)==1
                continue;
            else
                % Vertex RouteSet(i1,i) has not been considered
                % update NodeSet
                NodeSet=[NodeSet;RouteSet(i1,i)];
                %
                % given RouteSet, find out the possible routes to select at
                % Vertex RouteSet(i1,i)
                %
                % SetChoice: set of route (choices)
                % the set of routes: starts from vertex RouteSet(i1,i)
                % e.g., RouteSet(i1,i:(d+1))
                SetChoice=[];
                for i2=1:route_num % i2 is the route index
                    if RouteSet(i2,i)==RouteSet(i1,i)
                        SetChoice=[SetChoice;[RouteSet(i2,i:(d+1)) RouteSet(i2,(To+2):(To+7))]];
                    end
                end
                %
                % given Vertex RouteSet(i1,i) and visibility matrix, find out
                % the possible vertices that can be seen at RouteSet(i1,i)
                %
                for i2=1:(Num_oV-1)
                    % Visible_t:
                    % Visible_t(a,b): if Vertex a and Vertex b are visible
                    if (RouteSet(i1,i)==0)|(RouteSet(i1,i)>=Num_oV)
                        break;
                    else
                        if Visible_t(RouteSet(i1,i),i2)==0
                            continue;
                        else
                            % is Visible_t(RouteSet(i1,i),i2) included in routeV(,)
                            % is there a visible group? a suggested route
                            % i1: the route index; i: the time index;
                            % i2: vertex index.
                            % Is i2 on the suggested routes (routeV)?
                            [flag groupI]=ismember(i2,routeV(:,i));
                            if flag==1
                                % there is a visible group
                                % three points on the route: RouteSet(i1,i)
                                % Visible_t(RouteSet(i1,i),i2)
                                % routeV(groupI,)
                                Origin=RouteSet(i1,i);
                                Destination=routeV(groupI,i);
                                [dist,path1,pred] = shortestpath(BGobj,Origin,Destination,'directed',false,'method','BFS');
                                routeHerd=[path1(1:(dist+1)) routeV(groupI,(i+1):(d+1))];
                                % the number of people in the leading group
                                Lead_num=sum(GroupO(find(routeV(:,i)==i2),2));
                                % is the route routeHerd(1:(d+1-i)) included in
                                % SetChoice?
                                [flag1 routeI]=ismember(routeHerd(1:(d+2-i)),SetChoice(:,1:(d+2-i)),'rows');
                                if flag1==1
                                    SetChoice(routeI,d+5-i)=Lead_num;
                                    continue;
                                else
                                    % this route has not been included in SetChoice
                                    % the end of routeHerd
                                    End_routeH=routeHerd(d+2-i);
                                    % zf=0;zg=0; zh=1;zv=0;End_routeH;probability =1
                                    SetChoice=[SetChoice;routeHerd(1:(d+2-i)) 0,0,Lead_num,0,End_routeH,1];
                                    %
                                    %% rebuild the route and add this new route into
                                    % RouteSet
                                    RouteHerd=[];
                                    if i==1
                                        RouteHerd=routeHerd;
                                    else
                                        RouteHerd=[RouteSet(i1,1:(i-1)),routeHerd];
                                    end
                                    % update new (herding) routes in RouteSet
                                    RouteSet=[RouteSet;RouteHerd(1:(d+1)) 0,0,Lead_num,0,End_routeH,1];
                                    %
                                end % is the routeHerd(1:(d+1-i)) included in SetChoice?
                            end % is there a visible group? a suggested route (if flag==1)
                        end % if Visible_t(RouteSet(i1,i),i2)==0
                        %
                    end % the end of for loop: all possible routes are included in SetChoice and RouteSet
                    %
                    %% Given SetChoice and RouteSet, deriving/updating the probabilities
                    %
                    % Calculating probabilities from SetChoice
                    %
                    % Calculate the risk and blockage of the route
                    %
                    [Choice_num xx]=size(SetChoice);
                    % is there only one route to select?
                    % if yes, then the corresponding probability does not change
                    if Choice_num==1
                        continue;
                    else
                        % there are multiple routes to select
                        % PUtility: store the perceived utility for each route
                        PUtility=zeros(Choice_num,1);
                        % calculate the perceived utility for each route
                        for i2=1:Choice_num
                            Destination=SetChoice(i2,7+d-i);
                            Zb=PCostB(Destination);
                            Zr=PCostR(Destination);
                            for i3=1:(d-i+1)
                                % (starting) time point: i+i3-1
                                if SetChoice(i2,i3)==0
                                    continue;
                                else
                                    Zr=Zr+risk(SetChoice(i2,i3),i+i3-1);
                                    % if the group does not move at all
                                    if SetChoice(i2,i3+1)==0
                                        continue;
                                    else % !!! revised on 10/04/2018
                                        [EdgeN flag]=EdgeF(SetChoice(i2,i3),SetChoice(i2,i3+1),C);
                                        if EdgeN~=0
                                            Zb+lambdaV_t(SetChoice(i2,i3+1),i+i3)+lambda_t(EdgeN,i+i3-1)+q_t(EdgeN,i+i3-1)*ck/2;
                                        else
                                            Zb = Zb+lambdaV_t(SetChoice(i2,i3+1),i+i3);
                                        end   % !!! revised on 10/04/2018
                                    end
                                end
                            end
                            %
                            Zf=SetChoice(i2,d+3-i);
                            Zg=SetChoice(i2,d+4-i);
                            Zh=SetChoice(i2,d+5-i);
                            Zv=SetChoice(i2,d+6-i);
                            PUtility(i2)=Zr+Zb*betaB+Zg*betaG+Zh*betaH+Zv*betaV+Zg*Zv*betaGV+Zh*Zv*betaHV+Zh*Zs*betaHS;
                        end
                        % using the perceived utility, calculate the probablities
                        Exp_sum=0;
                        for i2=1:Choice_num
                            Exp_sum=Exp_sum+exp(-PUtility(i2));
                        end
                        for i2=1:Choice_num
                            SetChoice(i2,d+8-i)=exp(-PUtility(i2))/Exp_sum;
                        end
                        % update the corresponding probability
                        for i2=1:Choice_num
                            [xx routeN]=ismember(SetChoice(i2,1:(2+d-i)),RouteSet(:,i:(d+1)),'rows');
                            % update the probability in RouteSet
                            RouteSet(routeN,d+7)= RouteSet(routeN,d+7)*SetChoice(i2,d+8-i);
                        end
                    end
                    %
                end  % if the vertex (node) at t = i-1 has been considered
            end
            %
        end % end of the for loop: all vertices on Route RouteSet(:,i) are considered
    end
    %
        %
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % %%%%%%  Deriving probabilities of using routes   %%%%%%%%%%%%%
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %
    % Inputs: RouteSet, groupIndex, AllRoute  
    % outputs: AllRoute 
    %
    % AllRoute(the index of a group, the index of a possible route, T + probability)
    % AllRoute=zeros(Group_num,(Num_V+Group_num),(T+1));
    %
    [route_N xx]=size(RouteSet);
    for i=1:route_N
        Destination=RouteSet(i,d+6);
        if Destination==Num_V
            AllRoute(groupIndex,i,1:(d+1))=RouteSet(i,1:(d+1));
            AllRoute(groupIndex,i,(d+2):T)=zeros(1,(T-d-1));
            AllRoute(groupIndex,i,T+1)=RouteSet(i,d+7);
        else
            AllRoute(groupIndex,i,1:(d+1))=RouteSet(i,1:(d+1));
            flag=0;
            for i1=1:(1+d)
                if AllRoute(groupIndex,i,i1)==0
                    flag=1;
                    break;
                end
            end
            if flag==1
                v=AllRoute(groupIndex,i,i1-1);
                % HrouteV(v,(To+1):T)=OrouteV(1:(T-To));  % assuming T<Num_V
                AllRoute(groupIndex,i,(i1-1):(i1+T-To-2))= HrouteV(v,(To+1):T);
            else
                v=AllRoute(groupIndex,i,d+1);
                % HrouteV(v,(To+1):T)=OrouteV(1:(T-To));  % assuming T<Num_V
                AllRoute(groupIndex,i,(d+1):T)= HrouteV(v,(To+1):T);
            end
            AllRoute(groupIndex,i,T+1)=RouteSet(i,d+7);
        end
    end % reshape(AllRoute(1,:,:),[],(T+1))
    %
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % %%%%%%%%   Calculate expected desired flow rate and expected flow rates
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %
    % Inputs: GroupO, AllRoute, T, C, CapaN, CapaR,   
    % Function used: Gpath.m,
    % outputs: x 
    %
    %% set up the initial positions for all groups
    % Eq: the expected flow rates of different groups and different times;
    % Lcost: the Lagrangian; GroupO: people's distribution; routeV:
    % guidance, CapaN: capacities and familiarity,
    % Blc: the proportion of capacity when blocked (flow rates)
    %
    % x=zeros(group index, route index, vertex index,time index);
    % Old version: x=zeros(Num_V,T+1,cx);
    x=zeros(cx,(Num_V+cx),Num_V,T+1);
    % qdR=zeros(group index, route index, edge index,time index);
    % Old version: qdR=zeros(Num_E,T,cx);
    % the desired flow rate on each route for a particular group 
    qdR=zeros(cx,(Num_V+cx),Num_E,T);
    % Eq=zeros(group index, route index, edge index,time index);
    % Old version: Eq=zeros(Num_E,T,cx);
    % the flow rate on each route for a particular group 
    Eq=zeros(cx,(Num_V+cx),Num_E,T);
    % qd_sum: the sum of desired flow rate 
    qd_sum=zeros(Num_E,T); 
    % Eq_sum: the sum of expected flow rate (blockage at passages not rooms)
    Eq_sum=zeros(Num_E,T); 
    % Eq_new: the sum of expected flow rate (blockage at passages and rooms)
    p_pass=zeros(Num_E,T);
  %  Block_c=zeros(Num_V,T); % Block_c is a redundant vector
    Dr=ones(Num_E,T); % direction of flow on edges: "1" or "-1"
    for i1=1:cx
        locat = GroupO(i1,1); % locat=location
        for i4=1:(Num_V+cx)
            if AllRoute(i1,i4,1)~=0
                Proportion=AllRoute(i1, i4,(1+T)); % the probability of selecting the route
                x(i1,i4,locat,1)=GroupO(i1,2)*Proportion; % the number of people in group i1
            end
        end
    end % sum(sum(x(:,:,locat,1)))
    %
    % assuming every group follows the route suggested by guidance, deriving
    % deriving group dynamics/ Evaluating the expected desired flow rate
    for i=1:T    % i is the index of time
        % calculate the desired flow rates
        for i2=1:cx % i2 is the group index 
            for i4=1:(Num_V+cx) % i4 is route index
                if AllRoute(i2,i4,1)~=0
                    % AllRoute(the index of a group, the index of a possible route, T + probability)
                    pathV=Gpath(AllRoute(i2, i4,1:T),C);
                    %
                    for i3=1:Num_V
                        if pathV(i3)==0
                            break;
                        else
                            locat=AllRoute(i2, i4,i3);
                            %  locat=routeV(i2,i3);
                            edge_N=pathV(i3);           % edge number = pathV();
                            % qdR=zeros(group index, route index, edge index,time index);
                            % x=zeros(group index, route index, vertex index,time index);
                            % !!!Warning: Flow Direction is not represented                          
                            qdR(i2,i4,edge_N,i)=x(i2,i4,locat,i);
                            %
                        end
                    end
                else
                    break;
                end
            end
            if C(locat,edge_N)==1
                Dr(edge_N,i)=-1; % the opposite direction
            end
        end
        for i2=1:cx
            for i3=1:(Num_V+cx) % i3 is route index
                if AllRoute(i2,i3,1)~=0
                    for i4=1:Num_E  % i4 is index of passage
                        %  qdR=zeros(cx,(Num_V+cx),Num_E,T);
                        % [added] counterflow: abs(qdR)
                        qd_sum(i4,i)=qd_sum(i4,i)+abs(qdR(i2,i3,i4,i));
                    end
                else
                    break;
                end
            end
        end
        % calculate the expected flow rates
        for i4=1:Num_E
            Capacity=CapaN(i4,1);
            Qd=qd_sum(i4,i);
            if Qd>Capacity
                Eq_sum(i4,i)=(exp(-alpha/(Qd-Capacity)))*Capacity*Blc+(1-exp(-alpha/(Qd-Capacity)))*Capacity;
                p_pass(i4,i)=Eq_sum(i4,i)/qd_sum(i4,i);
            else
                Eq_sum(i4,i)=Qd;
                p_pass(i4,i)=1;
            end
%             %%%%%%%%%%%   consider room capacity %%%%%%%%%%
%             % i4: the index of an edge
%             [v1 v2]=fVex(C,i4);
%             %
%             if Dr(i4,i)==1 % the direction of Edge i4: 1 or -1?
%                 % v1: the starting vertex; v2: the ending vertex
%                 % the number of people at v2 at time step i
%                 num_x=sum(sum(x(:,:,v2,i)));
%                 %
%                 if v2<Num_oV  % if v2 is a real vertex
%                     % the possible number of people in v2 at next time step (i+1)
%                     if (num_x+Eq_sum(i4,i))>CapaR(v2)
%                       %  Block_c(v2,i)=1;
%                         Eq_new=CapaR(v2)-num_x;
%                         % update the probability of passing
%                         p_pass(i4,i)=Eq_new/qd_sum(i4,i);
%                     end
%                 end
%             else  % v2: the starting vertex; v1: the ending vertex
%                 % the number of people at v1 at time step i
%                 num_x=sum(sum(x(:,:,v1,i)));
%                 % x=zeros(group index, route index, vertex index,time index);
%                 %
%                 if v1<Num_oV  % if v2 is a real vertex
%                     % the possible number of people in v2 at next time step (i+1)
%                     if (num_x+Eq_sum(i4,i))>CapaR(v1)
%                        % Block_c(v1,i)=1;
%                         Eq_new=CapaR(v1)-num_x;
%                         % update the probability of passing
%                         p_pass(i4,i)=Eq_new/qd_sum(i4,i);
%                     end
%                 end
%             end
        end
        % update the states for different groups
        for i2=1:cx
            for i4=1:(Num_V+cx)  % i4 is route index
                if AllRoute(i2,i4,1)~=0
                    pathV=Gpath(AllRoute(i2, i4,1:T),C);
                    for i3=1:Num_V  % the i3th vertex on the route
                        if pathV(i3)==0
                            break;
                        else
                            %%% locat=routeV(i2,i3);
                            locat=AllRoute(i2,i4,i3);
                            % locat_n: the next vertex after "locat"
                            locat_n=AllRoute(i2,i4,i3+1);
                            %%% routeV(i2,i3+1);
                            edge_N=pathV(i3);                                                      
                            % the direction of the flow
                            % [v1 v2]=fVex(C,edge_N);
                            % !!! A bug will show up: if the group turn
                            % around on Edge edge_N
                            if i3==1
                                Eq(i2,i4,edge_N,i)=abs(qdR(i2,i4,edge_N,i)*p_pass(edge_N,i));
                            elseif edge_N_old==edge_N
                                Eq(i2,i4,edge_N,i)=0;
                            else
                                Eq(i2,i4,edge_N,i)=abs(qdR(i2,i4,edge_N,i)*p_pass(edge_N,i));
                            end
                            % % !!! Very tricky: the flow in and flow out       
                            if i3==1
                                % flow out from x(i2,i4,locat,i)
                                x(i2,i4,locat,i+1)=x(i2,i4,locat,i)-Eq(i2,i4,edge_N,i);
                                edge_N_old=edge_N; 
                            else
                                % flow in to x(i2,i4,locat_n,i+1)
                                edge_N_old=pathV(i3-1); 
                                % Eq(i2,i4,edge_N_old,i) has been computed
                                % Eq(i2,i4,edge_N_old,i)=qdR(i2,i4,edge_N_old,i)*p_pass(edge_N_old,i);
                                % Eq(i2,i4,edge_N,i) flows out from x(i2,i4,locat_n,i+1)
                                x(i2,i4,locat,i+1)=x(i2,i4,locat,i)+Eq(i2,i4,edge_N_old,i)-Eq(i2,i4,edge_N,i); 
                                x(i2,i4,locat_n,i+1)=x(i2,i4,locat_n,i)+Eq(i2,i4,edge_N,i);
                                edge_N_old=edge_N; 
                            end
                            %
                        end
                    end
                else
                    break;
                end
            end
        end
    end
    %
end
%% Update the guidance for the current group (not overlap in herding)
% Path_guide=PathV(k1,1:(1+To));
% Destination at the end of lookahead 
Destination=End_guide;
if Destination==Num_V
    routeV(groupIndex,1:(1+To))=Path_guide;
else
    routeV(groupIndex,1:(1+To))=Path_guide;
    routeV(groupIndex,(1+To):T)= HrouteV(End_guide,(To+1):T);
end
%
% %
% qd_sum=zeros(Num_E,T);
% for n1=1:Num_E
%     for n2=1:T
%         for n3=1:cx
%             for n4=1:(Num_V+cx)
%                 qd_sum(n1,n2)=qd_sum(n1,n2)+qdR(n3,n4,n1,n2);
%             end
%         end
%     end
% end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%   Update alpha_k,s^k,c^k,lambda^k    %%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Inputs: gnorm_tt, T, C, CapaN, alpha_tt, Stepsize_tt, iter_num, ck,
% qd_sum
% Function used: Gpath.m,
% outputs: gnorm_tt, alpha_tt, Stepsize_tt, iter_num, ck, q_t
%
%----------- Calculate g Funciton -----------------
% gnorm_t=gnorm_tt;
g_t=zeros(Num_E,T);    % Num_P: total number of paths
Capa_Matrix=CapaN(:,1)*ones(1,T);
g_t=qd_sum-Capa_Matrix;  %qd_sum=zeros(Num_E,T);
q_tt=abs(g_t); % the penalty for capacity constraints (passage capacity)
% calculating ||g||
gnorm_tt=0;
for i6=1:T
    for i7=1:Num_E
        gnorm_tt=gnorm_tt+g_t(i7,i6)^2;
    end
end
gnorm_tt=gnorm_tt^0.5;   %different from 01/06_2B
%--------------- Update LR Multiplier: lambda ------------------
%alpha_t=alpha_tt;    % update alpha
%Stepsize_t=Stepsize_tt;   % update stepsize (s^k) 
% "iter_num" is "k" (the number of iterations)
p=1-1/(iter_num^r1);     % update p value: p=1-1/(k^r), r1 is a constant, kth iteration
alpha_tt=1-1/(M*iter_num^p);  % alpha_{k+1}= 1-1/(M*k^p)
Stepsize_tt=(alpha_tt*Stepsize_t*gnorm_t)/(gnorm_tt); % the update c^(k+1)
ck1=ck*Beta; % update the parameter for penalty (absolute value)
for i=1:Num_E
    for j=1 :T
        lambda_tt(i,j)=lambda_t(i,j)+Stepsize_tt*g_t(i,j); % update multiplier changed to "+" stepsize*gradient
        if lambda_tt(i,j)<0
            lambda_tt(i,j)=0;  % from \hat{lambda} to its projection lambda
        end
    end
end
%