close all;
clear;clc;
fprintf('\nBuilding Models\n');% the initial status of different groups
Convertfds2Cood;
IncidenceM;
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%  Fire propagation and risks   %%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
Ft_Init=zeros(Num_oV,1);Ft_Init(23,1)=1;
s_F_origin=fire_prediction(Ft_Init, T+1, FireP); % calculating expected fire status
% assign (risk) probabilities if there are long passages between vertices 
s_F=AddVsF(s_F_origin,C,N_vex);   
% risk=zeros(Num_V,T+1); % initialize risk value
% the last vertex (Vertex 17) is outside the building, corresponding risk
% is 0
risk=[max(s_F(1:(Num_V-1),:),ones(Num_V-1,T+1)*r_c);zeros(1,(T+1))];     % calculating risk value % count groups' number: assign risk value for each group
%
% the set of vertices that are blocked by fire
%
Block_f=zeros(Num_V,T+1);
for i=1:(T+1)
    for i1=1:(Num_V)
        if risk(i1,i)>0.6 % 0.3 is assumed as the threshold
            Block_f(i1,i)=1; % Vertex i1 at time i is blocked by fire
        end
    end
end
%
%% Visibility affected by fire/smoke 
Visibility=zeros(To,Num_oV-1,Num_oV-1);
for i=1:To
   Visibility(i,:,:)=VisibleM;
   %
   Blocked=find(Block_f(:,i));
   % if there are vertices blocked by fire and smoke 
   if isempty(Blocked)==0
       for i1=1:length(Blocked)
           % the artificial vertex (on an edge) is not viewed as blocked 
           if Blocked(i1)>Num_oV  
               continue;
           else
               VertexOnFire=Blocked(i1);
           end
           Visibility(i,VertexOnFire,VertexOnFire)=0;
           %
           if VertexOnFire==3
               Visibility(i,2,6)=0;Visibility(i,6,2)=0;
               continue;
           end
           if VertexOnFire==12
               Visibility(i,3,16)=0;Visibility(i,16,3)=0;
               continue;
           end           
       end
   end
end   % reshape Visibility into "VisibleM?" When solving subproblems
% reshape(Visibility(i,:,:),(Num_V-1),(Num_V-1))
%%
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%  Set up groups: size and location   %%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% [Group GroupS GroupN GroupNO]=StanGro(Num_V,Num_oV,x,CapaP);% Group: the initial positions; GroupS: the number of evacuees; GroupN: the number of groups
% [GroupCal cx]=PreGroup(Group,GroupS); % GroupCal(i,:)=[the area index, the number of evacuees in that area]; cx is the number of areas in total
% GroupO=RestoreG(GroupCal,x,cx);   % GroupO: the original number of people in one group, the sequence: form large group to small group
% GroupO(i,1): the location of group i; GroupO(i,2): the number of
% occupants in group i
GroupO=PeoDistr;
%
[Group_num xx]=size(GroupO);
% AllRoute: the set of all possible routes used by different groups
% AllRoute(the index of a group, the index of a possible route, the vector of a route + probability)
AllRoute=zeros(Group_num,(Num_V+Group_num),(T+1));
% find out exit vertices
[ExitV ke]=fExit(Num_V,Num_E,C);  % ExitV is the vertices represent exits, ke is the total number of exits
% shortest path towards different exits
% note: there are ke nodes in BGobj (representing nodes for different exits)
% note: shortestpath(BGobj,Origin,Fami_Vertex,'directed',false,'method','BFS')
%        in shortestpath(BGobj, ...) different exits should be identified
%        differently
[SrouteV EdgeV]=heuriR(Num_V,Num_E,ExitV,ke,C,BGobj);
%
% %% Closeness between different groups 
% % if two groups are far away, there is no necessity to consider herding
% %
% % input: the initial location of each group 
% % 
% % Closeness[Group number, indices of close groups]
%
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%  Derive and Store:  %%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%  routes considered in different vertices %%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% routes considered at each vertex (w/o guidance and herding)
% RouteInMind(location, exit index?, elements)
%
RouteInMind=zeros((Num_oV),ke,(To+7));
    % RouteInMindi(To+2):Zf
    % RouteInMindi(To+3):Zg
    % RouteInMindi(To+4):Zh
    % RouteInMindi(To+5):Zv
    % RouteInMindi(To+6): Destination of the (partial) route 
    % RouteInMindi(To+7): Probability of selecting this route
    % Jt_r and Jt+b are functions of the vertex at RouteInMindi(To+6)
    % Zs is not included in this vector; defined at IncidenceM
%
% % RouteInMind(To+2):Zf
% % RouteInMind(To+3):Zg
% % RouteInMind(To+4):Zh
% % RouteInMind(To+5):Zv
% % RouteInMind(To+6):Jt_r ("risk-to-go" at t= T +1) 
% % RouteInMind(To+7):Jt_b ("blockage-to-go" at t= T +1)
%
% initializing the RouteInMind: familiarity and visibility 
for i1=1:(Num_oV-1)
    Origin=i1; 
    % the familiar route: Vertex 7 corresponds to Exit 3
    % 
    [fdist,fpath,pred] = shortestpath(BGobj,Origin,Fami_Vertex,'directed',false,'method','BFS');
    fpath=[fpath Num_V];
    fdist=fdist+1;
    %
    if fdist<To
        % if fdist < To, familiar path is the route to be considered
        RouteInMind(i1,Fami_Exit,1:(fdist+1))=fpath; % 3 corresponds to Exit 3
        % the destination of this route
        RouteInMind(i1,Fami_Exit,To+6)=Num_V;
    else
        % if fdist >= To, the vertex at VerS(d+1,:) is the ending point of the
        % route
        RouteInMind(i1,Fami_Exit,1:(To+1))=fpath(1:(To+1));
        % the destination of this route
        RouteInMind(i1,Fami_Exit,To+6)=fpath(To+1);
    end
    RouteInMind(i1,Fami_Exit,To+7)=1;
    % RouteInMind(To+2):Zf = 1
    RouteInMind(i1,Fami_Exit,(To+2))=1;
    %
    % the visible route to an exit
    % assign RouteInMind(To+5):Zv = 1
    for i=1:ke
        Dest=VisibleE(Origin,i);
        if (Dest~=0) 
            [vdist,vpath,pred] = shortestpath(BGobj,Origin,Dest,'directed',false,'method','BFS');
            vpath=[vpath Num_V];
            vdist=vdist+1;
            % 
            if vdist<To
                [flag routeIndex]=ismember(vpath,reshape(RouteInMind(i1,:,1:(vdist+1)),[],(vdist+1)),'rows');
                if flag==0 % if the visible route has not been considered
                    % if fdist < To, familiar path is the route to be considered
                    RouteInMind(i1,i,1:(vdist+1))=vpath; % 3 corresponds to Exit 3
                    % the destination of this route
                    RouteInMind(i1,i,To+6)=Num_V;
                    RouteInMind(i1,i,To+5)=1;
                else
                    RouteInMind(i1,routeIndex,To+5)=1;
                end
            else
                [flag routeIndex]=ismember(vpath(1:(d+1)),reshape(RouteInMind(i1,:,1:(d+1)),[],(d+1)),'rows');
                if flag==0 % if the visible route has not been considered
                    % if fdist >= To, the vertex at VerS(d+1,:) is the ending point of the
                    % route
                    RouteInMind(i1,i,1:(To+1))=vpath(1:(To+1));
                    % the destination of this route
                    %
                    % Error detected on 10/03/2018: "vpath" was "fpath" (incorrect)
                    RouteInMind(i1,i,To+6)=vpath(To+1);
                    RouteInMind(i1,i,To+5)=1;
                else
                    RouteInMind(i1,routeIndex,To+5)=1;
                end
            end
            RouteInMind(i1,i,To+7)=1;
        end
    end
end
% reshape(RouteInMind(1,:,:),[],(To+7))
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%   Initializing parameters    %%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Lagrange Multipliter: lambda Matrix
lambda_tt=zeros(Num_E,T);   % lambda at time t+1
g_t=zeros(Num_E, T);
Capa_Matrix=CapaN(:,1)*ones(1,T);
%
% Lagrange Multipliters for room capacities
% the related time indices: from t = 2 to T+1
lambdaV_t=zeros(Num_V,T);    % lambda at time t
% lambdaV_t(Num_V,:) = 0, there is no cost to stay outside
lambdaV_tt=zeros(Num_V,T);   % lambda at time t+1
g_tV=zeros(Num_V-1, T);
%  watch out the difference of dimension between lambdaV_t and CapaV_M
CapaV_Matrix=CapaV(:,1)*ones(1,T);  % CapaR should be tuned in NewStru(B,CapaP)
%
%-------- Set Parameters for LR --------------
threshold=0.001;  % ???
Stepsize_tt=0.8;    % ???
r1=0.9;M=2.0;      % r1 and M are constants in the SSG algorithm
ck1=0; % ck1 is the parameter in SAVLR. It is forced to be 0 (for SLR)
gnorm_tt=1;   %different from 01/06_2B
alpha_tt=1;   %different from 01/06_2B
Beta=1.1; % the beta to update penalty (absolute value)
% Initialize Lagrangian Multiplier % Note that lambda>=0
lambda_tt=lambda_tt+ones(size(lambda_tt))*0.1;  %initialization Lambda
q_tt=zeros(Num_E,T); % the absolute penalty in SAVLR 
routeV=zeros(Group_num,T); % the actual ending time is T-1
AllRoute=zeros(Group_num,(Num_V+Group_num),(T+1));
pathV=zeros(Group_num,Num_E);
p_pass=ones(Num_E,T);
qd_sum=zeros(Num_E,T);
counter1=0;flag1=0;flag_stop=0;flag2=0;
iter_num=1;
x=zeros(Group_num,(Num_V+Group_num),Num_V,T+1);
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%   Surrogate Absolute Value Lagrangian Relaxation  %%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
tic;
% Deriving heuristic routes for all vertices 
% Only risk caused by fire is considered (no blockages considered)
% Heuristics: After assigning risk values, using Dijkstra's algorithm
% Given the time steps lookahead (l or "d")
%
% calculate the initial solution at step 0
% LCost=0; % LCost = J
% Dr=zeros(Num_E,T);
% Block_c=zeros(Num_V,T);
% for i1=1:cx
%     [routeV, pathV, CostR, CostB]=dpsolve(GroupO(i1,1),GroupO(i1,2),i1, Num_oV, C, BGobj, ke,routeV, pathV, RouteInMind, To, SrouteV,EdgeV, StayC,betaG, betaH, betaF, betaB,risk,lambdaV_t,lambda_t, CapaN, p_pass, Block_f, Block_c);
% end
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%   Surrogate Absolute Value Lagrangian Relaxation  %%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
for iteration=1:100
    for j1=1:Group_num   % each group
        %
        % update the variaables in previous iteration 
        % update (has been incorporated): x, routeV, AllRoute
        %
        Stepsize_t=Stepsize_tt;
        gnorm_t=gnorm_tt;
        lambda_t=lambda_tt;
        alpha_t=alpha_tt;
        q_t=q_tt;
        ck=ck1;
        %
        % solve the group subproblem and update the variables  
        % v_I: GroupO(j1,1); groupIndex: j1; 
        % v_I=GroupO(j1,1); groupIndex=j1;
        [x,qd_sum,Stepsize_tt,gnorm_tt,lambda_tt,alpha_tt,ck1,q_tt,AllRoute,routeV]=SolveSubproblem(GroupO(j1,1),C,BGobj,j1,ke,routeV,RouteInMind,To,SrouteV,EdgeV, StayC,Blc,alpha, betaF,betaG, betaH,betaV, betaB,betaGV,betaHV,betaHS,risk,lambdaV_t,lambda_t,GroupO,CapaN, CapaR,r1, Beta, M, Stepsize_t, gnorm_t,alpha_t, ck, q_t, iter_num,AllRoute,x,Visibility,Zs,Num_oV,qd_sum);
        %
        % check the terminal condition
        % if Diff_lambda<0.00001
        Diff_lambda=0;   % terminate: reach the threshold?
        for i=1:Num_E
            for j=1 : T  %
                Diff_lambda = Diff_lambda+(lambda_tt(i,j)-lambda_t(i,j))^2;
            end
        end
        if Diff_lambda<0.001
            flag_stop=1;
            break;
        end
%         if abs(alpha_t-alpha_tt)<0.00001
%             flag_stop=1;
%             break;
%         end
        iter_num=iter_num+1;
        %
    end
    if flag_stop==1;
        break;
    end
end
toc;
% calculating the sum of risks and the duality gap 
xS=zeros(Num_V,T+1);
J=0;
for i=1:Num_V
    for j=1:(T+1)
        xS(i,j)=sum(sum(x(:,:,i,j)));
        J=J+xS(i,j)*risk(i,j);
    end
end
gap=sum(sum(qd_sum.*lambda_tt));
% LCostS=sum(CostR);
%
J
gap
% ConvertCood2fds;
% displayRoute(routeV,GroupNO,Num_oV,Num_V,ExitV,ke,A)
% GruF2 

% x_sum=zeros(Num_V,T+1);
% for n1=1:Num_V
%     for n2=1:(T+1)
%         for n3=1:cx
%             for n4=1:(Num_V+cx)
%                 x_sum(n1,n2)=x_sum(n1,n2)+x(n3,n4,n1,n2);
%             end
%         end
%     end
% end
% %
% qdr_sum=zeros(Num_E,T);
% for n1=1:Num_E
%     for n2=1:T
%         for n3=1:cx
%             for n4=1:(Num_V+cx)
%                 qdr_sum(n1,n2)=qdr_sum(n1,n2)+qdR(n3,n4,n1,n2);
%             end
%         end
%     end
% end